@@ -69,26 +69,17 @@ mantis_alerts <- function(
     fill_with_zero = fill_with_zero
   )
 
-  validate_df_to_inputspec(df, inputspec)
-  validate_alert_rules_to_inputspec(alert_rules, inputspec)
-
-  prepared_df <-
-    prepare_df(
-      df,
+  alert_results <-
+    prepare_and_run_alerts(
+      df = df,
       inputspec = inputspec,
+      alert_rules = alert_rules,
+      filter_results = filter_results,
       timepoint_limits = timepoint_limits,
       fill_with_zero = fill_with_zero
     )
 
-  results <-
-    run_alerts(
-      prepared_df = prepared_df,
-      inputspec = inputspec,
-      alert_rules = alert_rules,
-      filter_results = filter_results
-    )
-
-  results |>
+  alert_results |>
     dplyr::rename_with(
       .fn = item_cols_unprefix,
       .cols = dplyr::all_of(item_cols_prefix(inputspec$item_cols))
@@ -194,8 +185,6 @@ alert_rules <- function(
 ) {
   ars <- list(...)
 
-  # TODO: check only one rule of each type? Don't know if this is necessary
-
   err_validation <- character()
   is_alert_rule <- vapply(ars, is_alert_rule, logical(1))
   if (any(!is_alert_rule)) {
@@ -311,10 +300,10 @@ alert_missing <- function(
   )
 
   if (extent_type == "all") {
-    function_call <- quote(all(is.na(value)))
+    expression <- quote(all(is.na(value)))
     rule_description <- "All values are missing"
   } else if (extent_type == "any") {
-    function_call <- substitute(
+    expression <- substitute(
       sum(is.na(value)) >= x,
       list(x = extent_value)
     )
@@ -324,7 +313,7 @@ alert_missing <- function(
       " values are missing in total"
     )
   } else if (extent_type == "last") {
-    function_call <- substitute(
+    expression <- substitute(
       all(is.na(rev(value)[1:x])),
       list(x = extent_value)
     )
@@ -334,7 +323,7 @@ alert_missing <- function(
       " or more values are missing"
     )
   } else if (extent_type == "consecutive") {
-    function_call <- substitute(
+    expression <- substitute(
       {
         run_lengths <- rle(is.na(value))
         any(run_lengths$lengths[run_lengths$values] >= x)
@@ -349,7 +338,7 @@ alert_missing <- function(
 
   alert_rule(
     type = "missing",
-    function_call = function_call,
+    expression = expression,
     items = items,
     short_name = rule_short_name,
     description = rule_description
@@ -384,8 +373,6 @@ alert_equals <- function(
   rule_value,
   items = NULL
 ) {
-  # TODO: NEED TO THINK ABOUT WHAT TO DO WITH NAs
-  # currently just removing them but may want them in the extent_value
   validate_params_required(match.call())
   validate_params_type(
     match.call(),
@@ -408,7 +395,7 @@ alert_equals <- function(
   )
 
   if (extent_type == "all") {
-    function_call <- substitute(
+    expression <- substitute(
       all(value[!is.na(value)] == rv),
       list(rv = rule_value)
     )
@@ -417,7 +404,7 @@ alert_equals <- function(
       rule_value
     )
   } else if (extent_type == "any") {
-    function_call <- substitute(
+    expression <- substitute(
       sum(value[!is.na(value)] == rv) >= x,
       list(x = extent_value, rv = rule_value)
     )
@@ -428,7 +415,7 @@ alert_equals <- function(
       rule_value
     )
   } else if (extent_type == "last") {
-    function_call <- substitute(
+    expression <- substitute(
       all(rev(value[!is.na(value)])[1:x] == rv),
       list(x = extent_value, rv = rule_value)
     )
@@ -439,7 +426,7 @@ alert_equals <- function(
       rule_value
     )
   } else if (extent_type == "consecutive") {
-    function_call <- substitute(
+    expression <- substitute(
       {
         run_lengths <- rle(value[!is.na(value)])
         any(run_lengths$lengths[run_lengths$values == rv] >= x)
@@ -455,7 +442,7 @@ alert_equals <- function(
 
   alert_rule(
     type = "equals",
-    function_call = function_call,
+    expression = expression,
     items = items,
     short_name = rule_short_name,
     description = rule_description
@@ -509,7 +496,7 @@ alert_above <- function(
   )
 
   if (extent_type == "all") {
-    function_call <- substitute(
+    expression <- substitute(
       all(value[!is.na(value)] > rv),
       list(rv = rule_value)
     )
@@ -518,7 +505,7 @@ alert_above <- function(
       rule_value
     )
   } else if (extent_type == "any") {
-    function_call <- substitute(
+    expression <- substitute(
       sum(value[!is.na(value)] > rv) >= x,
       list(x = extent_value, rv = rule_value)
     )
@@ -529,7 +516,7 @@ alert_above <- function(
       rule_value
     )
   } else if (extent_type == "last") {
-    function_call <- substitute(
+    expression <- substitute(
       all(rev(value[!is.na(value)])[1:x] > rv),
       list(x = extent_value, rv = rule_value)
     )
@@ -540,7 +527,7 @@ alert_above <- function(
       rule_value
     )
   } else if (extent_type == "consecutive") {
-    function_call <- substitute(
+    expression <- substitute(
       {
         run_lengths <- rle(value[!is.na(value)] > rv)
         any(run_lengths$lengths[run_lengths$values] >= x)
@@ -556,7 +543,7 @@ alert_above <- function(
 
   alert_rule(
     type = "above",
-    function_call = function_call,
+    expression = expression,
     items = items,
     short_name = rule_short_name,
     description = rule_description
@@ -609,7 +596,7 @@ alert_below <- function(
   )
 
   if (extent_type == "all") {
-    function_call <- substitute(
+    expression <- substitute(
       all(value[!is.na(value)] < rv),
       list(rv = rule_value)
     )
@@ -618,7 +605,7 @@ alert_below <- function(
       rule_value
     )
   } else if (extent_type == "any") {
-    function_call <- substitute(
+    expression <- substitute(
       sum(value[!is.na(value)] < rv) >= x,
       list(x = extent_value, rv = rule_value)
     )
@@ -629,7 +616,7 @@ alert_below <- function(
       rule_value
     )
   } else if (extent_type == "last") {
-    function_call <- substitute(
+    expression <- substitute(
       all(rev(value[!is.na(value)])[1:x] < rv),
       list(x = extent_value, rv = rule_value)
     )
@@ -640,7 +627,7 @@ alert_below <- function(
       rule_value
     )
   } else if (extent_type == "consecutive") {
-    function_call <- substitute(
+    expression <- substitute(
       {
         run_lengths <- rle(value[!is.na(value)] < rv)
         any(run_lengths$lengths[run_lengths$values] >= x)
@@ -656,7 +643,7 @@ alert_below <- function(
 
   alert_rule(
     type = "below",
-    function_call = function_call,
+    expression = expression,
     items = items,
     short_name = rule_short_name,
     description = rule_description
@@ -726,7 +713,7 @@ alert_difference_above_perc <- function(
     )
   )
 
-  function_call <- substitute(
+  expression <- substitute(
     mean(rev(value)[cp], na.rm = TRUE) >
       (1 + rv / 100) * mean(rev(value)[pp], na.rm = TRUE),
     list(cp = current_period, pp = previous_period, rv = rule_value)
@@ -756,7 +743,7 @@ alert_difference_above_perc <- function(
 
   alert_rule(
     type = "diff_above_perc",
-    function_call = function_call,
+    expression = expression,
     items = items,
     short_name = rule_short_name,
     description = rule_description
@@ -826,7 +813,7 @@ alert_difference_below_perc <- function(
     )
   )
 
-  function_call <- substitute(
+  expression <- substitute(
     mean(rev(value)[cp], na.rm = TRUE) <
       (1 - rv / 100) * mean(rev(value)[pp], na.rm = TRUE),
     list(cp = current_period, pp = previous_period, rv = rule_value)
@@ -856,7 +843,7 @@ alert_difference_below_perc <- function(
 
   alert_rule(
     type = "diff_below_perc",
-    function_call = function_call,
+    expression = expression,
     items = items,
     short_name = rule_short_name,
     description = rule_description
@@ -868,7 +855,7 @@ alert_difference_below_perc <- function(
 #' Create a custom alert rule
 #'
 #' @section Details: `alert_custom()` - Specify a custom rule. The supplied
-#'   `function_call` is passed to `eval()` within a `dplyr::summarise()` after
+#'   `expression` is passed to `eval()` within a `dplyr::summarise()` after
 #'   grouping by the `item_cols` and ordering by the `timepoint_col`. Column
 #'   names that can be used explicitly in the expression are `value` and
 #'   `timepoint`, and which refer to the values in the `value_col` and
@@ -877,7 +864,7 @@ alert_difference_below_perc <- function(
 #' @param short_name Short name to uniquely identify the rule. Only include
 #'   alphanumeric, '-', and '_' characters.
 #' @param description Short description of what the rule checks for
-#' @param function_call Quoted expression containing the call to be evaluated
+#' @param expression Quoted expression containing the call to be evaluated
 #'   per item, that returns either `TRUE` or `FALSE`. Return value of `TRUE`
 #'   means alert result is "FAIL". See Details.
 #' @param items Named list with names corresponding to members of `item_cols`.
@@ -891,14 +878,14 @@ alert_difference_below_perc <- function(
 #'   alert_custom(
 #'     short_name = "my_rule_combo",
 #'     description = "Over 3 missing values and max value is > 10",
-#'     function_call = quote(
+#'     expression = quote(
 #'       sum(is.na(value)) > 3 && max(value, na.rm = TRUE) > 10
 #'     )
 #'   ),
 #'   alert_custom(
 #'     short_name = "my_rule_doubled",
 #'     description = "Last value is over double the first value",
-#'     function_call = quote(rev(value)[1] > 2 * value[1])
+#'     expression = quote(rev(value)[1] > 2 * value[1])
 #'   )
 #' )
 #'
@@ -907,24 +894,22 @@ alert_difference_below_perc <- function(
 alert_custom <- function(
   short_name,
   description,
-  function_call,
+  expression,
   items = NULL
 ) {
-  # TODO: Consider renaming function_call to quoted_expression
-
   validate_params_required(match.call())
   validate_params_type(
     match.call(),
     short_name = short_name,
     description = description,
-    function_call = function_call,
+    expression = expression,
     items = items
   )
 
   structure(
     list(
       type = "custom",
-      function_call = function_call,
+      expression = expression,
       short_name = short_name,
       description = description,
       items = items
@@ -959,7 +944,7 @@ is_alert_rules <- function(x) inherits(x, "mantis_alert_rules")
 #' Constructor for an alert_rule
 #'
 #' @param type type of rule
-#' @param function_call expression to pass to `eval()`, that returns either
+#' @param expression expression to pass to `eval()`, that returns either
 #'   `TRUE` or `FALSE`. Return value of `TRUE` means alert result is "FAIL"
 #' @param short_name a short computer-friendly name to uniquely identify the
 #'   rule
@@ -984,7 +969,7 @@ is_alert_rules <- function(x) inherits(x, "mantis_alert_rules")
 #' @noRd
 alert_rule <- function(
   type,
-  function_call,
+  expression,
   short_name,
   description,
   items = NULL
@@ -992,7 +977,7 @@ alert_rule <- function(
   structure(
     list(
       type = type,
-      function_call = function_call,
+      expression = expression,
       short_name = short_name,
       description = description,
       items = items
@@ -1074,8 +1059,6 @@ run_alert <- function(
 ) {
   timepoint <- NULL
 
-  # TODO: if it's a custom rule, wrap it in some error handling
-
   prepared_df |>
     restrict_items(items = alert_rule$items) |>
     dplyr::group_by(dplyr::across(dplyr::all_of(item_cols_prefix(
@@ -1085,10 +1068,53 @@ run_alert <- function(
     dplyr::summarise(
       alert_name = alert_rule$short_name,
       alert_description = alert_rule$description,
-      alert_result = ifelse(eval(alert_rule$function_call), "FAIL", "PASS")
+      alert_result = ifelse(eval(alert_rule$expression), "FAIL", "PASS")
     )
 }
 
+# -----------------------------------------------------------------------------
+#' Validate df and prepare alert results data frame
+#'
+#' @param df A data frame containing multiple time series in long format.
+#' @param inputspec [`inputspec()`] object specifying which columns in the
+#'   supplied `df` represent the "timepoint", "item", and "value" for the time
+#'   series.
+#' @param alert_rules [`alert_rules()`] object specifying conditions to test
+#' @param filter_results Only return rows where the alert result is in this
+#'   vector of values. Alert results can be "PASS", "FAIL", or "NA".
+#' @param timepoint_limits Set start and end dates for time period to include.
+#'   Defaults to min/max of `timepoint_col`. Can be either Date values or NAs.
+#' @param fill_with_zero Logical. Replace any missing or NA values with 0?
+#'   Useful when value_col is a record count.
+#' @return tibble
+#' @noRd
+prepare_and_run_alerts <- function(
+  df,
+  inputspec,
+  alert_rules,
+  filter_results = c("PASS", "FAIL", "NA"),
+  timepoint_limits = c(NA, NA),
+  fill_with_zero = FALSE
+) {
+  validate_df_to_inputspec(df, inputspec)
+  validate_alert_rules_to_inputspec(alert_rules, inputspec)
+
+  prepared_df <-
+    prepare_df(
+      df,
+      inputspec = inputspec,
+      timepoint_limits = timepoint_limits,
+      fill_with_zero = fill_with_zero
+    )
+
+  run_alerts(
+    prepared_df = prepared_df,
+    inputspec = inputspec,
+    alert_rules = alert_rules,
+    filter_results = filter_results
+  )
+}
+
 
 # -----------------------------------------------------------------------------
 #' Restrict prepared_df to specified items
