@@ -274,8 +274,6 @@ validate_param_byname <- function(
       error_message = "Values allowed are: sec, min, hour, day, week, month, quarter, year",
       error_contents_max_length = 100
     ),
-    # TODO: if they provide a colname that doesn't exist, is it better to error
-    # or ignore?
     "summary_cols" = validate_param(
       param_name = param_name,
       param_value = param_value,
@@ -451,7 +449,7 @@ validate_param_byname <- function(
       error_message = "Should only contain alphanumeric, '-', and '_' characters",
       error_contents_max_length = 255
     ),
-    "function_call" = validate_param(
+    "expression" = validate_param(
       param_name = param_name,
       param_value = param_value,
       allow_null = FALSE,
@@ -631,7 +629,7 @@ testfn_params_type <- function(
   previous_period = 4:9,
   short_name = "my_rule",
   description = "rule_description",
-  function_call = quote(all(is.na(value)))
+  expression = quote(all(is.na(value)))
 ) {
   if (missing(df)) {
     df <- data.frame("Fieldname" = 123)
@@ -699,7 +697,7 @@ testfn_params_type <- function(
     previous_period = previous_period,
     short_name = short_name,
     description = description,
-    function_call = function_call
+    expression = expression
   )
 }
 
